@@ -30,7 +30,7 @@
 // Author: Sanjay Ghemawat
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include "config.h"
 #endif
 
 #include <stdlib.h>
@@ -41,12 +41,11 @@
 #include <errno.h>
 #include <string>
 #include <algorithm>
-// We need this to compile the proper dll on windows/msys.  This is copied
-// from pcre_internal.h.  It would probably be better just to include that.
-#define PCRE_DEFINITION  /* Win32 __declspec(export) trigger for .dll */
-#include <pcre.h>
-#include "pcre_stringpiece.h"
+
+#include "pcrecpp_internal.h"
+#include "pcre.h"
 #include "pcrecpp.h"
+#include "pcre_stringpiece.h"
 
 
 namespace pcrecpp {
@@ -56,7 +55,7 @@ static const int kMaxArgs = 16;
 static const int kVecSize = (1 + kMaxArgs) * 3;  // results + PCRE workspace
 
 // Special object that stands-in for no argument
-Arg no_arg((void*)NULL);
+PCRECPP_EXP_DEFN Arg no_arg((void*)NULL);
 
 // If a regular expression has no error, its error_ field points here
 static const string empty_string;
@@ -332,13 +331,17 @@ bool RE::Replace(const StringPiece& rewrite,
 
 // Returns PCRE_NEWLINE_CRLF, PCRE_NEWLINE_CR, or PCRE_NEWLINE_LF.
 // Note that PCRE_NEWLINE_CRLF is defined to be P_N_CR | P_N_LF.
+// Modified by PH to add PCRE_NEWLINE_ANY and PCRE_NEWLINE_ANYCRLF.
+
 static int NewlineMode(int pcre_options) {
   // TODO: if we can make it threadsafe, cache this var
   int newline_mode = 0;
   /* if (newline_mode) return newline_mode; */  // do this once it's cached
-  if (pcre_options & (PCRE_NEWLINE_CRLF|PCRE_NEWLINE_CR|PCRE_NEWLINE_LF)) {
+  if (pcre_options & (PCRE_NEWLINE_CRLF|PCRE_NEWLINE_CR|PCRE_NEWLINE_LF|
+                      PCRE_NEWLINE_ANY|PCRE_NEWLINE_ANYCRLF)) {
     newline_mode = (pcre_options &
-                    (PCRE_NEWLINE_CRLF|PCRE_NEWLINE_CR|PCRE_NEWLINE_LF));
+                    (PCRE_NEWLINE_CRLF|PCRE_NEWLINE_CR|PCRE_NEWLINE_LF|
+                     PCRE_NEWLINE_ANY|PCRE_NEWLINE_ANYCRLF));
   } else {
     int newline;
     pcre_config(PCRE_CONFIG_NEWLINE, &newline);
@@ -348,6 +351,10 @@ static int NewlineMode(int pcre_options) {
       newline_mode = PCRE_NEWLINE_CR;
     else if (newline == 3338)
       newline_mode = PCRE_NEWLINE_CRLF;
+    else if (newline == -1)
+      newline_mode = PCRE_NEWLINE_ANY;
+    else if (newline == -2)
+      newline_mode = PCRE_NEWLINE_ANYCRLF;
     else
       assert("" == "Unexpected return value from pcre_config(NEWLINE)");
   }
@@ -362,7 +369,7 @@ int RE::GlobalReplace(const StringPiece& rewrite,
   int start = 0;
   int lastend = -1;
 
-  for (; start <= static_cast<int>(str->length()); count++) {
+  while (start <= static_cast<int>(str->length())) {
     int matches = TryMatch(*str, start, UNANCHORED, vec, kVecSize);
     if (matches <= 0)
       break;
@@ -377,9 +384,13 @@ int RE::GlobalReplace(const StringPiece& rewrite,
       // Note it's better to call pcre_fullinfo() than to examine
       // all_options(), since options_ could have changed bewteen
       // compile-time and now, but this is simpler and safe enough.
+      // Modified by PH to add ANY and ANYCRLF.
       if (start+1 < static_cast<int>(str->length()) &&
           (*str)[start] == '\r' && (*str)[start+1] == '\n' &&
-          NewlineMode(options_.all_options()) == PCRE_NEWLINE_CRLF) {
+          (NewlineMode(options_.all_options()) == PCRE_NEWLINE_CRLF ||
+           NewlineMode(options_.all_options()) == PCRE_NEWLINE_ANY ||
+           NewlineMode(options_.all_options()) == PCRE_NEWLINE_ANYCRLF)
+          ) {
         matchend++;
       }
       // We also need to advance more than one char if we're in utf8 mode.
@@ -463,7 +474,7 @@ int RE::TryMatch(const StringPiece& text,
     return 0;
   }
 
-  pcre_extra extra = { 0 };
+  pcre_extra extra = { 0, 0, 0, 0, 0, 0 };
   if (options_.match_limit() > 0) {
     extra.flags |= PCRE_EXTRA_MATCH_LIMIT;
     extra.match_limit = options_.match_limit();
@@ -607,23 +618,27 @@ bool Arg::parse_null(const char* str, int n, void* dest) {
 }
 
 bool Arg::parse_string(const char* str, int n, void* dest) {
+  if (dest == NULL) return true;
   reinterpret_cast<string*>(dest)->assign(str, n);
   return true;
 }
 
 bool Arg::parse_stringpiece(const char* str, int n, void* dest) {
+  if (dest == NULL) return true;
   reinterpret_cast<StringPiece*>(dest)->set(str, n);
   return true;
 }
 
 bool Arg::parse_char(const char* str, int n, void* dest) {
   if (n != 1) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<char*>(dest)) = str[0];
   return true;
 }
 
 bool Arg::parse_uchar(const char* str, int n, void* dest) {
   if (n != 1) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<unsigned char*>(dest)) = str[0];
   return true;
 }
@@ -672,6 +687,7 @@ bool Arg::parse_long_radix(const char* str,
   long r = strtol(str, &end, radix);
   if (end != str + n) return false;   // Leftover junk
   if (errno) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<long*>(dest)) = r;
   return true;
 }
@@ -689,6 +705,7 @@ bool Arg::parse_ulong_radix(const char* str,
   unsigned long r = strtoul(str, &end, radix);
   if (end != str + n) return false;   // Leftover junk
   if (errno) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<unsigned long*>(dest)) = r;
   return true;
 }
@@ -700,7 +717,8 @@ bool Arg::parse_short_radix(const char* str,
   long r;
   if (!parse_long_radix(str, n, &r, radix)) return false; // Could not parse
   if (r < SHRT_MIN || r > SHRT_MAX) return false;       // Out of range
-  *(reinterpret_cast<short*>(dest)) = r;
+  if (dest == NULL) return true;
+  *(reinterpret_cast<short*>(dest)) = static_cast<short>(r);
   return true;
 }
 
@@ -711,7 +729,8 @@ bool Arg::parse_ushort_radix(const char* str,
   unsigned long r;
   if (!parse_ulong_radix(str, n, &r, radix)) return false; // Could not parse
   if (r > USHRT_MAX) return false;                      // Out of range
-  *(reinterpret_cast<unsigned short*>(dest)) = r;
+  if (dest == NULL) return true;
+  *(reinterpret_cast<unsigned short*>(dest)) = static_cast<unsigned short>(r);
   return true;
 }
 
@@ -722,6 +741,7 @@ bool Arg::parse_int_radix(const char* str,
   long r;
   if (!parse_long_radix(str, n, &r, radix)) return false; // Could not parse
   if (r < INT_MIN || r > INT_MAX) return false;         // Out of range
+  if (dest == NULL) return true;
   *(reinterpret_cast<int*>(dest)) = r;
   return true;
 }
@@ -733,6 +753,7 @@ bool Arg::parse_uint_radix(const char* str,
   unsigned long r;
   if (!parse_ulong_radix(str, n, &r, radix)) return false; // Could not parse
   if (r > UINT_MAX) return false;                       // Out of range
+  if (dest == NULL) return true;
   *(reinterpret_cast<unsigned int*>(dest)) = r;
   return true;
 }
@@ -753,11 +774,14 @@ bool Arg::parse_longlong_radix(const char* str,
   long long r = strtoq(str, &end, radix);
 #elif defined HAVE_STRTOLL
   long long r = strtoll(str, &end, radix);
+#elif defined HAVE__STRTOI64
+  long long r = _strtoi64(str, &end, radix);
 #else
 #error parse_longlong_radix: cannot convert input to a long-long
 #endif
   if (end != str + n) return false;   // Leftover junk
   if (errno) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<long long*>(dest)) = r;
   return true;
 #endif   /* HAVE_LONG_LONG */
@@ -780,11 +804,14 @@ bool Arg::parse_ulonglong_radix(const char* str,
   unsigned long long r = strtouq(str, &end, radix);
 #elif defined HAVE_STRTOLL
   unsigned long long r = strtoull(str, &end, radix);
+#elif defined HAVE__STRTOI64
+  unsigned long long r = _strtoui64(str, &end, radix);
 #else
 #error parse_ulonglong_radix: cannot convert input to a long-long
 #endif
   if (end != str + n) return false;   // Leftover junk
   if (errno) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<unsigned long long*>(dest)) = r;
   return true;
 #endif   /* HAVE_UNSIGNED_LONG_LONG */
@@ -802,6 +829,7 @@ bool Arg::parse_double(const char* str, int n, void* dest) {
   double r = strtod(buf, &end);
   if (end != buf + n) return false;   // Leftover junk
   if (errno) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<double*>(dest)) = r;
   return true;
 }
@@ -809,6 +837,7 @@ bool Arg::parse_double(const char* str, int n, void* dest) {
 bool Arg::parse_float(const char* str, int n, void* dest) {
   double r;
   if (!parse_double(str, n, &r)) return false;
+  if (dest == NULL) return true;
   *(reinterpret_cast<float*>(dest)) = static_cast<float>(r);
   return true;
 }
